#<profile4xi profile = "31"/>
# -------------------------------------------------------------------------------------------------
# find4cl.py finds eight (8) text string in PyOpenCL programs
#            the history at the end of this module can be reused for new searches
#<program4xi program = "find4cl.py">
#<abstract4xi abstract = "finds eight (8) text string in a file"/>
#<refer4xi reference = "Studio4Xi"/>
#<note4xi note = "Reviewing all tests, when the output file is too large for editor"/>
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'find4cl' ]
#"
#/>



# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os,sys,string
import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from time4xi import time4xi
from version4xi import VersionKB4Xi
from version4xi import ROOT4XI
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi Globals                            
# -------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global kb2find
global keys2find
global max2find
global path2opencl4xi
global profile2find
global rare2find
#"
#"/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
kb2find = {}
rare2find = 'alaloydatatajonoa'
path2opencl4xi = VersionKB4Xi['opencl'] # defines folder to be searched with all its subfolders

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<define4xi definition = "
#
profile2find = '.py'                    # '.py'=Python, '.cpp'=C++, '.hpp'=headers, '.c'=C

kb2find[1]='build_ext'                 #
kb2find[2]='build_py'                  # 
# kb2find[3]=rare2find  
# kb2find[4]=rare2find        
# kb2find[5]=rare2find         
# kb2find[6]=rare2find          
# kb2find[7]=rare2find           
# kb2find[8]=rare2find     

keys2find = kb2find.keys()
#"
#/>



# -------------------------------------------------------------------------------------------------
#<function4xi function = "find4xi" usage = "
#   find anchor references
#"> 
# -------------------------------------------------------------------------------------------------
def find4xi(icepFullName):
    global path2opencl4xi
    global profile2find

    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------    
    iceName=os.path.splitext(icepFullName)
    
    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------      
    count2passed = 0
    count2found = 0
    if iceName[1] == profile2find:          
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        print ('EXAMINING FILE: ' + icepFullName)
        # print ('iceName[0]: ' + iceName[0])
        # print ('iceName[1]: ' + iceName[1])              
        
        iceInput = iceName[0] + profile2find 
        input_file = open(iceInput)
        while 1:
            l_in = input_file.readline()                               # process an input line
            if not l_in:
                break                                                  # all lines have been read
            else:
                for key2find in keys2find:
                    if kb2find[key2find] == rare2find:         
                        break                                          # nothing to be find anymore
                    else:    
                        i_xi2=str.find(l_in,kb2find[key2find])
                        if i_xi2 == -1:
                            count2passed = count2passed + 1                # no match
                        else: 
                            count2found = count2found + 1              # match
                            print ('FOUND: ' + icepFullName + ':')
                            # 
                            print ('-LINE: ' + l_in)
                            # 

        print ('icepFullName: ' + icepFullName)
        print ('count2passed: ')
        print (count2passed)
        print ('count2found: ')
        print (count2found)
        input_file.close()                                             # close file
    else:
        print ('SKIPPED FILE: ' + icepFullName)
        # print ('iceName[0]: ' + iceName[0])
        # print ('iceName[1]: ' + iceName[1])        
        
            
#</function4xi>
    

# =================================================================================================
#<class4xi class = "DirectoryWalker" base = "None" child = "None" synopsis = "a forward iterator that traverses a directory tree" usage = "
#   to review all tests"
#>
# =================================================================================================
class DirectoryWalker:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes DirectoryWalker"> 
    # ---------------------------------------------------------------------------------------------
    def __init__(self, directory):
        self.stack = [directory]
        self.files = []
        self.index = 0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getitem" usage = "getting item"> 
    # ---------------------------------------------------------------------------------------------
    def __getitem__(self, index):
        while 1:
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------          
            try:
                # --------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------             
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # --------------------------------------------------------------------------------- 
                # pop next directory from stack
                # ---------------------------------------------------------------------------------             
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # --------------------------------------------------------------------------------- 
                # got a filename
                # ---------------------------------------------------------------------------------             
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                return fullname
    #</methodize4xi>
#</class4xi>

     
# -------------------------------------------------------------------------------------------------
#<function4xi function = "main" usage = "
#   reviewing all tests
#">                           
# -------------------------------------------------------------------------------------------------
def main():
    global path2opencl4xi
    global profile2find
    
    # --------------------------------------------------------------------------------------------- 
    #
    # ---------------------------------------------------------------------------------------------        
    print('=' *70)
    print("Find4cl [Rel 7.21] for ", path2opencl4xi, " with profile: ", profile2find)
    print('=' *70)    
    
    try:
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------     
        for ice_file in DirectoryWalker(path2opencl4xi):
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------     
            if os.path.isdir(ice_file):
                print ("Directory: " + ice_file + '\n')
            if os.path.isfile(ice_file):
                #print "Input file: " + ice_file + '\n'
                find4xi(ice_file)
        # --------------------------------------------------------------------------------------------- 
        #
        # --------------------------------------------------------------------------------------------- 
        print('=' *75)
        print("Done Find4cl [Rel 7.21] for ", path2opencl4xi, " with profile: ", profile2find)
        print('=' *75)
    except:
        print("*** ERROR in Find4cl [Rel 7.21] for ", path2opencl4xi, " with profile: ", profile2find) 
        # *** ERROR in Find4cl [Rel 7.21] for  ..//..//..//..//opencl4xi//  with profile:  .py    
#</function4xi>
    
        
# -------------------------------------------------------------------------------------------------
if __name__ == '__main__':
    main()        
#</program4xi>


# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
# ===========================================================================
# Find4cl [Rel 7.21] for  ..//..//..//..//pyopencl4xi// - with profile: py
# ===========================================================================
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\arithmetic_container.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\batchjob.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\convergence.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\datatable.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\debug.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\decorator.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\diskdict.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\lex.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\log.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\mpi.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\mpiwrap.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\obj_array.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\prefork.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\socket4xi.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\spatial_btree.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\stopwatch.py
# icepFullName: ..//..//..//..//pyopencl4xi//pytools\__init__.py
# 
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\arrayimpl.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\mpiunittest.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\runalltest.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\spawn_child.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_cco_buf.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_cco_obj.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_cco_vec.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_comm.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_datatype.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_doc.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_environ.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_errhandler.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_errorcode.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_exceptions.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_file.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_fortran.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_grequest.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_group.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_info.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_mem.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_message.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_msgzero.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_op.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_p2p_buf.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_p2p_obj.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_pack.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_request.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_rma.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_spawn.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_status.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_subclass.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_threads.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\test\test_win.py
# 
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\helloworld.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\wrap-swig\test.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\wrap-f2py\test.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\wrap-cython\test.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\wrap-boost\test.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\threads\sendrecv.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\spawning\cpi-master.py
# icepFullName: ..//..//..//..//pyopencl4xi//mpi4py\demo\spawning\cpi-worker.py
# 
# ===============================================================================
# Done Find4cl [Rel 7.21] for  ..//..//..//..//pyopencl4xi//- with profile: py
# ===============================================================================
#"
#>
    

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of find4cl.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

